// See COPYING for license information.

#include <cstdlib>
#include <vector>
#include <fstream>
#include <iostream>
#include "defs.h"
#include "CDownloader.h"
#include "CConfigFile.h"
#include "CMirrorHandler.h"

CMirrorHandler::CMirrorHandler(const bool& choose)
{
	if ( choose )	mirror_choose();
	mirror_reload();
}

void CMirrorHandler::mirror_reload()
{
	// Get the current mirror in the config.
	std::string	_curmirror = CConfigFile::Instance().GetValue("CurrentMirror")[0];

	if ( !_curmirror.find("local://") ) {	// Local mirror.
		std::cout << "This must be a local mirror, looking for PACKAGES.TXT...\n";
		system(std::string("rm -f " + TMPFILE("PACKAGES.TXT")).c_str());
		system(std::string("cp -vf " + _curmirror.substr(8) + "/PACKAGES.TXT " + TMPFILE("PACKAGES.TXT") +
					"|| echo \"None found, but that's usually fine.\"").c_str());
		return;
	}

	// Try download the compressed package list.
	CDownloader	_dler;
	if ( !_dler.Download(_curmirror + "/PACKAGES.TXT.gz", TMPFILE("PACKAGES.TXT.gz")) ) {	// Second try sans .gz.
		std::cerr << "Trying to download the uncompressed package list instead...\n";
		_dler.Download(_curmirror + "/PACKAGES.TXT", TMPFILE("PACKAGES.TXT"));
	} else
		system(std::string("gzip -df " + TMPFILE("PACKAGES.TXT.gz")).c_str());
	system(std::string("touch -c " + TMPFILE("PACKAGES.TXT")).c_str());	// Update timestamp.
}

void CMirrorHandler::mirror_choose()
{	// This function is for choosing a mirror.
	std::string			_curmirror = CConfigFile::Instance().GetValue("CurrentMirror")[0];
	std::vector<std::string>	_mirrors = CConfigFile::Instance().GetValue("Mirror");

	// Display all the mirrors including which one is currently being used.
	for ( unsigned int i = 0; i < _mirrors.size(); i++ )
		std::cout << "#" <<  i + 1 << "\t" << _mirrors[i] << std::endl;
	for ( unsigned int i = 0; i < _mirrors.size(); i++ ) {
		if ( _mirrors[i] == _curmirror ) {
			std::cout << std::endl << "Current mirror is #" << i + 1 << std::endl << std::endl;
			std::cout << "Enter a mirror number, press Ctrl+D to refresh the same one,\nor press Ctrl+C to cancel: #";
			std::cout.flush();
		}
	}

	// Let the user choose a new one.
	unsigned int	_newmirror = 0;
	std::cin >> _newmirror;
	std::cout << std::endl;
	if ( _newmirror > 0 && _newmirror <= _mirrors.size() )
		_curmirror = _mirrors[_newmirror - 1];	// Convert mirror number choice to URL string.
	std::vector<std::string>	_tmplist(1, _curmirror);
	CConfigFile::Instance().SetValue("CurrentMirror", _tmplist);
	WRITELOG("Mirror changed to " + _curmirror);
}
