// See COPYING for license information.

#include <algorithm>
#include <fstream>
#include <vector>
#include "defs.h"
#include "CParser.h"
#include "CPkgDetails.h"

CPkgDetails::CPkgDetails(std::vector<std::string>& packages)
{
	// Read the database into memory so that we can
	// find the package on the mirror or locally.
	CParser		_parser;
	package_map&	_db = _parser.Packages;

	for ( unsigned int i = 0; i < packages.size(); i++ ) {
		// Convert to lower case first.
		std::transform(packages[i].begin(), packages[i].end(), packages[i].begin(), tolower);

		// Check the mirror first.
		if ( !_db[packages[i]].mirror_version.empty() ) {
			std::cout << VT_COLOR_UPGRADE << "\nPackage '" << _db[packages[i]].name << "-"
				<< _db[packages[i]].mirror_version << "' found on mirror.\n" << VT_COLOR_NORMAL;
			info_from_file(_db[packages[i]].name, _db[packages[i]].mirror_version, TMPFILE("PACKAGES.TXT"));
		} else if ( !_db[packages[i]].installed_version.empty() ) {	// Check installed.
			std::cout << VT_COLOR_UPGRADE <<  "\nPackage '" << _db[packages[i]].name << "-"
				<< _db[packages[i]].installed_version << "' found installed.\n" << VT_COLOR_NORMAL;
			info_from_file(_db[packages[i]].name,
					_db[packages[i]].installed_version,
					std::string("/var/log/packages/" + _db[packages[i]].name + "-" + _db[packages[i]].installed_version));
		} else	// Not found.
			std::cerr << VT_COLOR_DOWNGRADE << "Package '" << packages[i] << "' could not be found on mirror or installed.\n" << VT_COLOR_NORMAL;
	}
}

void CPkgDetails::info_from_file(const std::string& package, const std::string& version, const std::string& file)
{	// Read the package info from a file.
	std::ifstream			_ifdb;
	std::string			_instr;
	bool				_are_we_there_yet = false;
	std::vector<std::string>	_desc;
	_ifdb.open(file.c_str(), std::ios_base::in);
	while ( _ifdb.is_open() && !_ifdb.eof() ) {
		getline(_ifdb, _instr);
		if ( !_instr.find("PACKAGE NAME:") && (int)_instr.find(package + "-" + version) > 12 )
			_are_we_there_yet = true;
		else if ( !_instr.find("PACKAGE NAME:") )
			_are_we_there_yet = false;
		if ( !_are_we_there_yet )	continue;

		if ( !_instr.find("PACKAGE LOCATION:") && file.find("/var/log/packages") )
			std::cout << "Location:\t\t" << _instr.substr(_instr.find("./") + 2) << std::endl;
		if ( !_instr.find("COMPRESSED") || !_instr.find("PACKAGE SIZE (compressed)") )
			std::cout << "Compressed size:\t" << ltrim(_instr.substr(_instr.find(':') + 1)) << std::endl;
		if ( !_instr.find("UNCOMPRESSED") || !_instr.find("PACKAGE SIZE (uncompressed)") )
			std::cout << "Uncompressed size:\t" << ltrim(_instr.substr(_instr.find(':') + 1)) << std::endl;
		if ( !_instr.find("PACKAGE REQUIRED:") )
			std::cout << "Dependencies:\t\t" << ltrim(_instr.substr(_instr.find(':') + 1)) << std::endl;
		if ( !_instr.find(package + ":") )
			_desc.push_back(ltrim(_instr.substr(_instr.find(':') + 1)));
	}
	_ifdb.close();

	// Print out the description.
	
	std::cout << "Description:\n";
	for ( unsigned int i = 0; i < _desc.size(); i++ ) {
		if ( _desc[i].empty() && i != 1 )	continue;
		std::cout << _desc[i] << std::endl;
	}
}

std::string CPkgDetails::ltrim(std::string str)
{	// Remove all space prefixing a string.
	while ( !str.find(' ') )
		str.erase(0, 1);
	return str;
}
